﻿using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.UI;
using LoongCAD.UI.Component;
using LoongCAD.Util;
using System;
using System.Windows.Forms;

namespace Example_DetailComponent1
{
    public class PluginMain : DetailCompPlugin
    {
        private readonly Guid m_nameGuid = new Guid(System.Reflection.Assembly.GetExecutingAssembly().GetName().Name);
        private Form_PluginMain m_propsForm = null;
        public PluginMain()
        {

        }
        public override bool StandAlone => true;
        public override Guid Guid => m_nameGuid;
        public override string CompName => "细部";
        public override string CompGroup => "细部";
        public override int CompNumber => 002;

#if DEBUG
        public override bool DebugMode => true;
#else
        public override bool DebugMode => false;
#endif
        /// <summary>
        /// 执行组件过程，必需实现此方法
        /// </summary>
        /// <param name="compInst"></param>
        /// <param name="defaultId"></param>
        /// <returns></returns>
        public override bool OnInstanceRun(ComponentBase compInst, long defaultId)
        {
            if (compInst == null || !(compInst is DetailComponent))
                return false;

            // 1. 获取组件实例的属性
            DObject compProps = compInst.UIProps;
            if (compProps == null)
            {
                compProps = new DObject();
            }

            // 2. 初始化参数处理器，此过程包含以下操作:
            // * 初始化参数值
            // * 初始化参数空值表
            // * 初始化参数内置默认值表
            // * 组合参数默认值
            ParamsHandler paramsHandler = new ParamsHandler();
            paramsHandler.ParamValues = compProps;
            paramsHandler.InitializeDefaultValues(GetDefaultsData(defaultId));
            // 3. 初始化组件上下文，此过程执行以下操作
            var compContext = ComponentInstanceContext.CreateInstance(compInst as DetailComponent, paramsHandler);
            if (compContext == null)
                return false;
            // 4. 执行组件的每一个功能
            {
            }
            return true;
        }
        /// <summary>
        /// 显示组件的属性窗，必须实现此方法
        /// </summary>
        /// <param name="ownerForm">组件窗口所属主窗口</param>
        /// <returns>是否显示成功</returns>
        /// <remarks><b>其他组件可直接复制此方法</b></remarks>
        public override bool OnShowPluginPropsForm(Form ownerForm)
        {
#if false
            if (m_propsForm == null) {
                m_propsForm = new Form_PluginMain(this);
                m_propsForm.Owner = ownerForm;
            }

            m_propsForm.ShowPluginProps();
#else
            Form_ComponentProps pluginForm = GetMainForm(ownerForm);
            pluginForm.ShowPluginProps();
#endif
            return true;
        }

        /// <summary>
        /// 显示组件实例的属性窗，必须实现此方法
        /// </summary>
        /// <param name="instObj">组件实例对象</param>
        /// <param name="defId">组件默认值ID</param>
        /// <param name="ownerForm">组件窗口所属主窗口</param>
        /// <returns>是否显示成功</returns>
        /// <remarks><b>其他组件可直接复制此方法</b></remarks>
        public override bool OnShowInstancePropsForm(ComponentBase instObj, long defId, Form ownerForm)
        {
            if (instObj == null)
                return false;

#if false
            if (m_propsForm == null) {
                m_propsForm = new Form_PluginMain(this);
                m_propsForm.Owner = ownerForm;
            }

            m_propsForm.ShowInstanceProps(instObj,
                new ParamsHandler().InitializeDefaultValues(this.GetDefaultsData(defId)));
            return true;
#else
            Form_ComponentProps pluginForm = GetMainForm(ownerForm);
            pluginForm.ShowInstanceProps(instObj,
                new ParamsHandler().InitializeDefaultValues(GetDefaultsData(defId)));
            return true;
#endif
        }

        private Form_PluginMain GetMainForm(Form ownerForm)
        {
            Form_PluginMain pluginForm = FormPool.GetPluginForm(Guid, "main") as Form_PluginMain;
            if (pluginForm == null)
            {
                pluginForm = new Form_PluginMain(this);
                pluginForm.Owner = ownerForm;
                FormPool.PushPluginForm(Guid, "main", pluginForm);
            }

            return pluginForm;
        }
    }
}
